Method: Weak::Map::StrongSecondaryKeys.usable?

Defined in:
lib/weak/map/strong_secondary_keys.rb

.usable?Bool

Checks if this strategy is usable for the current Ruby version.

Returns:

  • (Bool)

    always true to indicate that this strategy should be usable with any Ruby implementation which provides an ObjectSpace::WeakMap.



60
61
62
# File 'lib/weak/map/strong_secondary_keys.rb', line 60

def self.usable?
  true
end