Method: Weak::Map#compare_by_identity?

Defined in:
lib/weak/map.rb

#compare_by_identity?true

Returns always true since we always compare elements by their object identity.

Returns:

  • (true)

    always true since we always compare elements by their object identity



363
364
365
# File 'lib/weak/map.rb', line 363

def compare_by_identity?
  true
end