Method: Weak::Set#compare_by_identity?

Defined in:
lib/weak/set.rb

#compare_by_identity?true

Returns always true since we always compare elements by their object identity.

Returns:

  • (true)

    always true since we always compare elements by their object identity



418
419
420
# File 'lib/weak/set.rb', line 418

def compare_by_identity?
  true
end